// Processes.cpp
//
// Copyright (c), ..  (andrei.glukhoedov@gmail.com)
//

#include <Windows.h>
#include <tchar.h>
#include <strsafe.h>

// ------------------------------------------------------------------------------------------------
BOOL WaitProcessById(DWORD dwProcessId, DWORD dwMilliseconds, LPDWORD lpExitCode)
{
    //  
    HANDLE hProcess = OpenProcess(SYNCHRONIZE|PROCESS_QUERY_INFORMATION, FALSE, dwProcessId);

    if (NULL == hProcess)
    {
        return FALSE; //    
    } // if

    //   
    WaitForSingleObject(hProcess, dwMilliseconds);

    if (NULL != lpExitCode)
    {
        //    
        GetExitCodeProcess(hProcess, lpExitCode);
    } // if

    //   
    CloseHandle(hProcess);

    return TRUE;
} // WaitProcessById

// ------------------------------------------------------------------------------------------------
BOOL EnumProcessesInJob(HANDLE hJob, ULONG_PTR *lpidProcess, DWORD cb, LPDWORD lpcbNeeded)
{
    //    ,
    //     lpidProcess
    DWORD nCount = cb / sizeof(ULONG_PTR);

    if (NULL != lpidProcess && nCount > 0)
    {
        //     ( )
        //     
        DWORD cbJobPIL = sizeof(JOBOBJECT_BASIC_PROCESS_ID_LIST) + (nCount - 1) * sizeof(ULONG_PTR);

        //   
        JOBOBJECT_BASIC_PROCESS_ID_LIST *pJobPIL = (JOBOBJECT_BASIC_PROCESS_ID_LIST *)malloc(cbJobPIL);

        if (NULL != pJobPIL)
        {
            //    ,
            //     
            pJobPIL->NumberOfAssignedProcesses = nCount;

            //    
            BOOL bRet = QueryInformationJobObject(hJob, JobObjectBasicProcessIdList, pJobPIL, cbJobPIL, NULL);

            if (FALSE != bRet)
            {
                //   
                nCount = pJobPIL->NumberOfProcessIdsInList;
                //     
                CopyMemory(lpidProcess, pJobPIL->ProcessIdList, nCount*sizeof(ULONG_PTR));

                //     ( ),
                //     
                if (NULL != lpcbNeeded)
                    *lpcbNeeded = nCount * sizeof(ULONG_PTR);
            } // if

            free(pJobPIL); //  
            return bRet;
        } // if
    } // if

    return FALSE;
} // EnumProcessesInJob

// ------------------------------------------------------------------------------------------------
BOOL StartGroupProcessesAsJob(HANDLE hJob, LPCTSTR lpszCmdLine[], DWORD nCount, BOOL bInheritHandles, DWORD dwCreationFlags)
{
    // ,      
    BOOL bInJob = FALSE;
    IsProcessInJob(GetCurrentProcess(), NULL, &bInJob);

    if (FALSE != bInJob) //   (!)
    {
        //     ,
        //      

        JOBOBJECT_BASIC_LIMIT_INFORMATION jobli = { 0 };

        QueryInformationJobObject(NULL,
            JobObjectBasicLimitInformation, &jobli, sizeof(jobli), NULL);

        DWORD dwLimitMask = JOB_OBJECT_LIMIT_BREAKAWAY_OK | JOB_OBJECT_LIMIT_SILENT_BREAKAWAY_OK;

        if ((jobli.LimitFlags & dwLimitMask) == 0)
        {
            /*    
                   */
            return FALSE;
        } // if
    } // if

    //  ...

    TCHAR szCmdLine[MAX_PATH];
    STARTUPINFO si = { sizeof(STARTUPINFO) };
    PROCESS_INFORMATION pi = { 0 };

    for (DWORD i = 0; i < nCount; ++i)
    {
        StringCchCopy(szCmdLine, MAX_PATH, lpszCmdLine[i]);

        //   ,
        //     
        BOOL bRet = CreateProcess(NULL, szCmdLine, NULL, NULL,
            bInheritHandles, dwCreationFlags|CREATE_SUSPENDED|CREATE_BREAKAWAY_FROM_JOB, NULL, NULL, &si, &pi);

        if (FALSE != bRet)
        {
            //     
            AssignProcessToJobObject(hJob, pi.hProcess);
            //     
            ResumeThread(pi.hThread);
            //     
            CloseHandle(pi.hThread);
            //    
            CloseHandle(pi.hProcess);
        } // if
    } // for

    return TRUE;
} // StartGroupProcessesAsJob
