// SampleRegLibrary.cpp
//

#include <Windows.h>
#include <tchar.h>

#include "SampleRegLibrary.h"

// ------------------------------------------------------------------------------------------------
LSTATUS RegGetValueType(HKEY hKey, LPCTSTR lpValueName, LPDWORD lpdwType)
{
    //     
    return RegQueryValueEx(hKey, lpValueName, NULL, lpdwType, NULL, NULL);
} // RegGetValueType

// ------------------------------------------------------------------------------------------------
LSTATUS RegGetValueQWORD(HKEY hKey, LPCTSTR lpValueName, PULONG64 lpdwData)
{
    DWORD dwType;
    //     
    LSTATUS lStatus = RegQueryValueEx(hKey, lpValueName, NULL, &dwType, NULL, NULL);

    if (ERROR_SUCCESS == lStatus && REG_QWORD == dwType)
    {
        //    ( )
        DWORD cb = sizeof(ULONG64);
        //   
        lStatus = RegQueryValueEx(hKey, lpValueName, NULL, NULL, (LPBYTE)lpdwData, &cb);
    } // if
    else if (ERROR_SUCCESS == lStatus)
    {
        lStatus = ERROR_UNSUPPORTED_TYPE; //   
    } // if

    return lStatus;
} // RegGetValueQWORD

// ------------------------------------------------------------------------------------------------
LSTATUS RegGetValueDWORD(HKEY hKey, LPCTSTR lpValueName, LPDWORD lpdwData)
{
    DWORD dwType;
    //     
    LSTATUS lStatus = RegQueryValueEx(hKey, lpValueName, NULL, &dwType, NULL, NULL);

    if (ERROR_SUCCESS == lStatus && REG_DWORD == dwType)
    {
        //    ( )
        DWORD cb = sizeof(DWORD);
        //   
        lStatus = RegQueryValueEx(hKey, lpValueName, NULL, NULL, (LPBYTE)lpdwData, &cb);
    } // if
    else if (ERROR_SUCCESS == lStatus)
    {
        lStatus = ERROR_UNSUPPORTED_TYPE; //   
    } // if

    return lStatus;
} // RegGetValueDWORD

// ------------------------------------------------------------------------------------------------
LSTATUS RegGetValueSZ(HKEY hKey, LPCTSTR lpValueName, LPTSTR lpszData, DWORD cch, LPDWORD lpcchNeeded)
{
    DWORD dwType;
    //     
    LSTATUS lStatus = RegQueryValueEx(hKey, lpValueName, NULL, &dwType, NULL, NULL);

    if (ERROR_SUCCESS == lStatus && REG_SZ == dwType)
    {
        //    ( )
        DWORD cb = cch * sizeof(TCHAR);
        //   
        lStatus = RegQueryValueEx(hKey, lpValueName, NULL, NULL, (LPBYTE)lpszData, &cb);

        if (NULL != lpcchNeeded)
            *lpcchNeeded = cb / sizeof(TCHAR);
    } // if
    else if (ERROR_SUCCESS == lStatus)
    {
        lStatus = ERROR_UNSUPPORTED_TYPE; //   
    } // if

    return lStatus;
} // RegGetValueSZ

// ------------------------------------------------------------------------------------------------
LSTATUS RegGetValueExpandSZ(HKEY hKey, LPCTSTR lpValueName, LPTSTR lpszData, DWORD cch, LPDWORD lpcchNeeded)
{
    DWORD dwType;
    //     
    LSTATUS lStatus = RegQueryValueEx(hKey, lpValueName, NULL, &dwType, NULL, NULL);

    if (ERROR_SUCCESS == lStatus && REG_EXPAND_SZ == dwType)
    {
        //    ( )
        DWORD cb = cch * sizeof(TCHAR);
        //   
        lStatus = RegQueryValueEx(hKey, lpValueName, NULL, NULL, (LPBYTE)lpszData, &cb);

        if (NULL != lpcchNeeded)
            *lpcchNeeded = cb / sizeof(TCHAR);
    } // if
    else if (ERROR_SUCCESS == lStatus)
    {
        lStatus = ERROR_UNSUPPORTED_TYPE; //   
    } // if

    return lStatus;
} // RegGetValueExpandSZ

// ------------------------------------------------------------------------------------------------
LSTATUS RegGetValueMultiSZ(HKEY hKey, LPCTSTR lpValueName, LPTSTR lpszData, DWORD cch, LPDWORD lpcchNeeded)
{
    DWORD dwType;
    //     
    LSTATUS lStatus = RegQueryValueEx(hKey, lpValueName, NULL, &dwType, NULL, NULL);

    if (ERROR_SUCCESS == lStatus && REG_MULTI_SZ == dwType)
    {
        //    ( )
        DWORD cb = cch * sizeof(TCHAR);
        //   
        lStatus = RegQueryValueEx(hKey, lpValueName, NULL, NULL, (LPBYTE)lpszData, &cb);

        if (NULL != lpcchNeeded)
            *lpcchNeeded = cb / sizeof(TCHAR);
    } // if
    else if (ERROR_SUCCESS == lStatus)
    {
        lStatus = ERROR_UNSUPPORTED_TYPE; //   
    } // if

    return lStatus;
} // RegGetValueMultiSZ

// ------------------------------------------------------------------------------------------------
LSTATUS RegGetValueBinary(HKEY hKey, LPCTSTR lpValueName, LPBYTE lpData, DWORD cb, LPDWORD lpcbNeeded)
{
    DWORD dwType;
    //     
    LSTATUS lStatus = RegQueryValueEx(hKey, lpValueName, NULL, &dwType, NULL, NULL);

    if (ERROR_SUCCESS == lStatus && REG_BINARY == dwType)
    {
        //   
        lStatus = RegQueryValueEx(hKey, lpValueName, NULL, NULL, lpData, &cb);

        if (NULL != lpcbNeeded) *lpcbNeeded = cb;
    } // if
    else if (ERROR_SUCCESS == lStatus)
    {
        lStatus = ERROR_UNSUPPORTED_TYPE; //   
    } // if

    return lStatus;
} // RegGetValueBinary

// ------------------------------------------------------------------------------------------------
LSTATUS RegSetValueQWORD(HKEY hKey, LPCTSTR lpValueName, ULONG64 dwData)
{
    //   
    return RegSetValueEx(hKey, lpValueName, 0, REG_QWORD, (LPCBYTE)&dwData, sizeof(ULONG64));
} // RegSetValueQWORD

// ------------------------------------------------------------------------------------------------
LSTATUS RegSetValueDWORD(HKEY hKey, LPCTSTR lpValueName, DWORD dwData)
{
    //   
    return RegSetValueEx(hKey, lpValueName, 0, REG_DWORD, (LPCBYTE)&dwData, sizeof(DWORD));
} // RegSetValueDWORD

// ------------------------------------------------------------------------------------------------
LSTATUS RegSetValueSZ(HKEY hKey, LPCTSTR lpValueName, LPCTSTR lpszData)
{
    //     ( )
    DWORD cb = (_tcslen(lpszData) + 1) * sizeof(TCHAR);
    //   
    return RegSetValueEx(hKey, lpValueName, 0, REG_SZ, (LPCBYTE)lpszData, cb);
} // RegSetValueSZ

// ------------------------------------------------------------------------------------------------
LSTATUS RegSetValueExpandSZ(HKEY hKey, LPCTSTR lpValueName, LPCTSTR lpszData)
{
    //     ( )
    DWORD cb = (_tcslen(lpszData) + 1) * sizeof(TCHAR);
    //   
    return RegSetValueEx(hKey, lpValueName, 0, REG_EXPAND_SZ, (LPCBYTE)lpszData, cb);
} // RegSetValueExpandSZ

// ------------------------------------------------------------------------------------------------
LSTATUS RegSetValueMultiSZ(HKEY hKey, LPCTSTR lpValueName, LPCTSTR lpszData)
{
    //    
    DWORD cb = sizeof(TCHAR);
    for (LPCTSTR p = lpszData; p[0] != TEXT('\0');)
    {
        DWORD cch = _tcslen(p) + 1;

        cb += cch * sizeof(TCHAR);
        p  += cch;
    } // if

    //   
    return RegSetValueEx(hKey, lpValueName, 0, REG_MULTI_SZ, (LPCBYTE)lpszData, cb);
} // RegSetValueMultiSZ

// ------------------------------------------------------------------------------------------------
LSTATUS RegSetValueBinary(HKEY hKey, LPCTSTR lpValueName, LPCBYTE lpData, DWORD cb)
{
    //   
    return RegSetValueEx(hKey, lpValueName, 0, REG_BINARY, (LPCBYTE)lpData, cb);
} // RegSetValueBinary
