// Sample1.cpp
//
// Copyright (c), ..  (andrei.glukhoedov@gmail.com)
//

#include <Windows.h>
#include <tchar.h>
#include <strsafe.h>
#include <stdio.h>
#include <locale.h>
#include <LMCons.h>

#include <SecurityLibrary.h>

#define SAFE_FREE(ptr) if (NULL != (ptr)) LocalFree(ptr)

// ,        SID  
void PrintSID(PSID pSid);
// ,           
void PrintPrivilegesAndAccountRights(LSA_HANDLE lsahPolicy, PSID pSid);

// ------------------------------------------------------------------------------------------------
int _tmain()
{
    _tsetlocale(LC_ALL, TEXT(""));

    PSID pDomainSid = NULL; // SID  

    //       
    LSA_HANDLE lsahPolicy = OpenLocalPolicy(POLICY_LOOKUP_NAMES);

    // //// //

    TCHAR szComputerName[MAX_COMPUTERNAME_LENGTH + 1] = TEXT("");
    DWORD cch = _countof(szComputerName);

    //   
    BOOL bRet = GetComputerName(szComputerName, &cch);

    if (FALSE != bRet)
    {
        //  SID 
        bRet = GetAccountSID(szComputerName, &pDomainSid);

        if ((FALSE != bRet) && (NULL != pDomainSid))
        {
            //    SID 
            PrintSID(pDomainSid);
        } // if
    } // if

    // //// //

    TCHAR szUserName[UNLEN + 1] = TEXT("");
    cch = _countof(szUserName);

    //    
    bRet = GetUserName(szUserName, &cch);

    if (FALSE != bRet)
    {
        PSID pSid = NULL;

        //  SID  
        bRet = GetAccountSID(szUserName, &pSid);

        if ((FALSE != bRet) && (NULL != pSid))
        {
            //    SID  
            PrintSID(pSid);

            //         
            if (NULL != lsahPolicy) PrintPrivilegesAndAccountRights(lsahPolicy, pSid);

            //   
            LocalFree(pSid);
        } // if
    } // if

    // //// //

    constexpr WELL_KNOWN_SID_TYPE aWellKnown[] = {
        WinNullSid,
        WinWorldSid,
        WinConsoleLogonSid,
        WinDialupSid,
        WinNetworkSid,
        WinBatchSid,
        WinInteractiveSid,
        WinServiceSid,
        WinAnonymousSid,
        WinAuthenticatedUserSid,
        WinLocalSystemSid,
        WinLocalServiceSid,
        WinNetworkServiceSid,
        WinAccountAdministratorSid,
        WinAccountGuestSid,
        WinAccountDomainAdminsSid,
        WinAccountDomainUsersSid,
        WinAccountDomainGuestsSid,
        WinAccountComputersSid,
        WinAccountControllersSid,
        WinBuiltinAdministratorsSid,
        WinBuiltinUsersSid,
        WinBuiltinGuestsSid,
        WinBuiltinPowerUsersSid,
        WinBuiltinAccountOperatorsSid,
        WinBuiltinPrintOperatorsSid,
        WinBuiltinRemoteDesktopUsersSid,
        WinBuiltinNetworkConfigurationOperatorsSid
    };

    for (int i = 0; i < _countof(aWellKnown); ++i)
    {
        PSID pSid = NULL;

        //    SID
        bRet = GetWellKnownSID(aWellKnown[i], pDomainSid, &pSid);

        if ((FALSE != bRet) && (NULL != pSid))
        {
            //    SID  
            PrintSID(pSid);

            //       
            if (NULL != lsahPolicy) PrintPrivilegesAndAccountRights(lsahPolicy, pSid);

            //   
            LocalFree(pSid);
        } // if
    } // for

    // //// //

    //   
    SAFE_FREE(pDomainSid);

    //     
    LsaClose(lsahPolicy);
} // _tmain

// ------------------------------------------------------------------------------------------------
void PrintSID(PSID pSid)
{
    LPTSTR lpszSid = NULL, lpszAccountName = NULL;

    //  SID  
    ConvertSidToStringSid(pSid, &lpszSid);

    //       SID
    GetAccountName(pSid, &lpszAccountName);

    if ((NULL != lpszAccountName) && (NULL != lpszSid))
    {
        //       SID  
        _tprintf(TEXT("%s (%s)\n\n"), lpszAccountName, lpszSid);
    } // if

    //   
    SAFE_FREE(lpszSid);
    SAFE_FREE(lpszAccountName);
} // PrintSID

// ------------------------------------------------------------------------------------------------
void PrintPrivilegesAndAccountRights(LSA_HANDLE lsahPolicy, PSID pSid)
{
    PLSA_UNICODE_STRING UserRights; //    
    ULONG nCountOfRights; //    

    //     
    NTSTATUS ntsResult = LsaEnumerateAccountRights(lsahPolicy, pSid, &UserRights, &nCountOfRights);

    if (LSA_SUCCESS(ntsResult))
    {
        _tprintf(TEXT("\t   :\n"));

        for (ULONG i = 0; i < nCountOfRights; ++i)
        {
            LPCWSTR sUserRight = UserRights[i].Buffer;

            _tprintf(TEXT("\t  %2d. %s"), (i + 1), sUserRight);

            //   

            TCHAR szDisplayName[256];
            DWORD cch = _countof(szDisplayName), dwLangId;

            BOOL bRet = LookupPrivilegeDisplayName(NULL, sUserRight, szDisplayName, &cch, &dwLangId);

            if (FALSE != bRet)
            {
                _tprintf(TEXT(" (%s)"), szDisplayName);
            } // if
            else
            {
                constexpr LPCTSTR aNameRight[20] = {
                    SE_INTERACTIVE_LOGON_NAME, TEXT("   "),
                    SE_DENY_INTERACTIVE_LOGON_NAME, TEXT("  "),
                    SE_NETWORK_LOGON_NAME, TEXT("    "),
                    SE_DENY_NETWORK_LOGON_NAME, TEXT("     "),
                    SE_BATCH_LOGON_NAME, TEXT("    "),
                    SE_DENY_BATCH_LOGON_NAME, TEXT("      "),
                    SE_SERVICE_LOGON_NAME, TEXT("   "),
                    SE_DENY_SERVICE_LOGON_NAME, TEXT("     "),
                    SE_REMOTE_INTERACTIVE_LOGON_NAME, TEXT("        "),
                    SE_DENY_REMOTE_INTERACTIVE_LOGON_NAME, TEXT("        ")
                };

                for (int j = 0; j < _countof(aNameRight); j += 2)
                {
                    if (_tcscmp(aNameRight[j], sUserRight) == 0)
                    {
                        _tprintf(TEXT(" (%s)"), aNameRight[j + 1]);
                        break;
                    } // if
                } // for
            } // else

            _tprintf(TEXT("\n"));
        } // for

        _tprintf(TEXT("\n"));

        //     
        LsaFreeMemory(UserRights);
    } // if
} // PrintPrivilegesAndAccountRights
