// SecurityLibrary.cpp
//
// Copyright (c), ..  (andrei.glukhoedov@gmail.com)
//

#include <Windows.h>
#include <tchar.h>
#include <strsafe.h>

#include "SecurityLibrary.h"

// ----------------------------------------------------------------------------------------------
BOOL GetOwnerName(PSECURITY_DESCRIPTOR pSD, LPTSTR *lpName)
{
    PSID pSid;
    BOOL bDefaulted;

    //  SID 
    BOOL bRet = GetSecurityDescriptorOwner(pSD, &pSid, &bDefaulted);

    if (FALSE != bRet)
    {
        //     
        bRet = GetAccountName(pSid, lpName);
    } // if

    return bRet;
} // GetOwnerName

// ----------------------------------------------------------------------------------------------
BOOL GetEntriesFromDacl(PSECURITY_DESCRIPTOR pSD, PULONG pcCountOfEntries, PEXPLICIT_ACCESS *pListOfEntries)
{
    PACL pDacl;
    BOOL bDaclPresent, bDaclDefaulted;

    //  DACL
    BOOL bRet = GetSecurityDescriptorDacl(pSD, &bDaclPresent, &pDacl, &bDaclDefaulted);

    if (FALSE != bRet && FALSE != bDaclPresent)
    {
        //    DACL
        DWORD dwResult = GetExplicitEntriesFromAcl(pDacl, pcCountOfEntries, pListOfEntries);
        bRet = (ERROR_SUCCESS == dwResult) ? TRUE : FALSE;
    } // if
    else
    {
        *pcCountOfEntries = 0; //  0 
    } // else

    return bRet;
} // GetExplicitEntriesFromDacl

// ----------------------------------------------------------------------------------------------
BOOL DeleteEntryFromDalc(PSECURITY_DESCRIPTOR pSD, DWORD dwIndex)
{
    PACL pDacl;
    BOOL bDaclPresent, bDaclDefaulted;

    //  DACL
    BOOL bRet = GetSecurityDescriptorDacl(pSD, &bDaclPresent, &pDacl, &bDaclDefaulted);

    if (FALSE != bRet)
    {
        //    DACL
        bRet = DeleteAce(pDacl, dwIndex);
    } // if

    return bRet;
} // DeleteEntryFromDalc

// ----------------------------------------------------------------------------------------------
BOOL GetObjectSecurityDescriptor(HANDLE hObject, SECURITY_INFORMATION RequestedInformation, PSECURITY_DESCRIPTOR *ppSD)
{
    DWORD cb = 0;

    //    
    GetKernelObjectSecurity(hObject, RequestedInformation, NULL, 0, &cb);

    //     
    PSECURITY_DESCRIPTOR pSD = (PSECURITY_DESCRIPTOR)LocalAlloc(LMEM_FIXED, cb);
    if (NULL == pSD) return FALSE;

    //   
    BOOL bRet = GetKernelObjectSecurity(hObject, RequestedInformation, pSD, cb, &cb);

    if (FALSE != bRet)
    {
        *ppSD = pSD; //    
    } // if
    else
    {
        LocalFree(pSD); //   
    } // else

    return bRet;
} // GetObjectSecurityDescriptor

// ----------------------------------------------------------------------------------------------
BOOL SetObjectSecurityInfo(HANDLE hObject, LPCTSTR lpAccountOwner, ULONG cCountOfEntries, PEXPLICIT_ACCESS pListOfEntries, BOOL bMergeEntries)
{
    SECURITY_DESCRIPTOR sd; //  

    PSID pOwner = NULL; //   SID 
    PACL pNewDacl = NULL; //    DACL

    //   
    BOOL bRet = InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);

    // ///// //

    if (FALSE != bRet && NULL != lpAccountOwner)
    {
        //  SID   
        bRet = GetAccountSID(lpAccountOwner, &pOwner);

        //   SID   
        if (FALSE != bRet)
            bRet = SetSecurityDescriptorOwner(&sd, pOwner, FALSE);
    } // if

    // ///// //

    if (FALSE != bRet && cCountOfEntries > 0 && NULL != pListOfEntries)
    {
        PSECURITY_DESCRIPTOR pOldSD = NULL;
        PACL pOldDacl = NULL; //     DACL

        BOOL bDaclDefaulted = FALSE;

        if (FALSE != bMergeEntries)
        {
            bRet = GetObjectSecurityDescriptor(hObject, DACL_SECURITY_INFORMATION, &pOldSD);

            if (FALSE != bRet)
            {
                BOOL bDaclPresent;
                //  DACL
                GetSecurityDescriptorDacl(pOldSD, &bDaclPresent, &pOldDacl, &bDaclDefaulted);
            } // if
        } // if

        //   DACL
        DWORD dwResult = SetEntriesInAcl(cCountOfEntries, pListOfEntries, pOldDacl, &pNewDacl);
        bRet = (ERROR_SUCCESS == dwResult) ? TRUE : FALSE;

        //   DACL   
        if (FALSE != bRet)
            bRet = SetSecurityDescriptorDacl(&sd, TRUE, pNewDacl, bDaclDefaulted);

        //  
        if (NULL != pOldSD) LocalFree(pOldSD);
    } // if

    // ///// //

    if (FALSE != bRet)
    {
        SECURITY_INFORMATION si = 0;
        if (NULL != pOwner) si |= OWNER_SECURITY_INFORMATION;
        if (NULL != pNewDacl) si |= DACL_SECURITY_INFORMATION;

        //   
        bRet = SetKernelObjectSecurity(hObject, si, &sd);
    } // if

    //  ,     SID
    if (NULL != pOwner) LocalFree(pOwner);
    //  ,     DACL
    if (NULL != pNewDacl) LocalFree(pNewDacl);

    return bRet;
} // SetObjectSecurityInfo

// ----------------------------------------------------------------------------------------------
BOOL GetFileSecurityDescriptor(LPCTSTR lpFileName, SECURITY_INFORMATION RequestedInformation, PSECURITY_DESCRIPTOR *ppSD)
{
    DWORD cb = 0;

    //    
    GetFileSecurity(lpFileName, RequestedInformation, NULL, 0, &cb);

    //     
    PSECURITY_DESCRIPTOR pSD = (PSECURITY_DESCRIPTOR)LocalAlloc(LMEM_FIXED, cb);
    if (NULL == pSD) return FALSE;

    //   
    BOOL bRet = GetFileSecurity(lpFileName, RequestedInformation, pSD, cb, &cb);

    if (FALSE != bRet)
    {
        *ppSD = pSD; //    
    } // if
    else
    {
        LocalFree(pSD); //   
    } // else

    return bRet;
} // GetFileSecurityDescriptor

// ----------------------------------------------------------------------------------------------
BOOL SetFileSecurityInfo(LPCTSTR lpFileName, LPCTSTR lpAccountOwner, ULONG cCountOfEntries, PEXPLICIT_ACCESS pListOfEntries, BOOL bMergeEntries)
{
    SECURITY_DESCRIPTOR sd; //  

    PSID pOwner = NULL; //   SID 
    PACL pNewDacl = NULL; //    DACL

    //   
    BOOL bRet = InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);

    // ///// //

    if (FALSE != bRet && NULL != lpAccountOwner)
    {
        //  SID   
        bRet = GetAccountSID(lpAccountOwner, &pOwner);

        //   SID   
        if (FALSE != bRet)
            bRet = SetSecurityDescriptorOwner(&sd, pOwner, FALSE);
    } // if

    // ///// //

    if (FALSE != bRet && cCountOfEntries > 0 && NULL != pListOfEntries)
    {
        PSECURITY_DESCRIPTOR pOldSD = NULL;
        PACL pOldDacl = NULL; //     DACL

        BOOL bDaclDefaulted = FALSE;

        if (FALSE != bMergeEntries)
        {
            bRet = GetFileSecurityDescriptor(lpFileName, DACL_SECURITY_INFORMATION, &pOldSD);

            if (FALSE != bRet)
            {
                BOOL bDaclPresent;

                //  DACL
                GetSecurityDescriptorDacl(pOldSD, &bDaclPresent, &pOldDacl, &bDaclDefaulted);
            } // if
        } // if

        //   DACL
        DWORD dwResult = SetEntriesInAcl(cCountOfEntries, pListOfEntries, pOldDacl, &pNewDacl);
        bRet = (ERROR_SUCCESS == dwResult) ? TRUE : FALSE;

        //   DACL   
        if (FALSE != bRet)
            bRet = SetSecurityDescriptorDacl(&sd, TRUE, pNewDacl, bDaclDefaulted);

        //  
        if (NULL != pOldSD) LocalFree(pOldSD);
    } // if

    // ///// //

    if (FALSE != bRet)
    {
        SECURITY_INFORMATION si = 0;
        if (NULL != pOwner) si |= OWNER_SECURITY_INFORMATION;
        if (NULL != pNewDacl) si |= DACL_SECURITY_INFORMATION;

        //   
        bRet = SetFileSecurity(lpFileName, si, &sd);
    } // if

    //  ,     SID
    if (NULL != pOwner) LocalFree(pOwner);
    //  ,     DACL
    if (NULL != pNewDacl) LocalFree(pNewDacl);

    return bRet;
} // SetFileSecurityInfo
