// Token.cpp
//
// Copyright (c), ..  (andrei.glukhoedov@gmail.com)
//

#include <Windows.h>
#include <tchar.h>
#include <strsafe.h>

// ------------------------------------------------------------------------------------------------
HANDLE OpenProcessTokenByProcessId(DWORD dwProcessId, DWORD dwDesiredAccess)
{
    HANDLE hToken = NULL;

    //   
    HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, dwProcessId);

    if (NULL != hProcess)
    {
        //    
        OpenProcessToken(hProcess, dwDesiredAccess, &hToken);
        //   
        CloseHandle(hProcess);
    } // if

    return hToken;
} // OpenProcessTokenByProcessId

// ------------------------------------------------------------------------------------------------
HANDLE OpenToken(DWORD dwDesiredAccess)
{
    HANDLE hToken = NULL;

    //     
    BOOL bRet = OpenThreadToken(GetCurrentThread(), dwDesiredAccess, FALSE, &hToken);

    if (FALSE == bRet &&
        GetLastError() == ERROR_NO_TOKEN) //  (!) :      
    {
        //    
        OpenProcessToken(GetCurrentProcess(), dwDesiredAccess, &hToken);
    } // if

    return hToken;
} // OpenToken

// ------------------------------------------------------------------------------------------------
HANDLE OpenUserToken(LPCTSTR lpszUsername, LPCTSTR lpszDomain, LPCTSTR lpszPassword, DWORD dwLogonType,
    DWORD dwDesiredAccess, PSECURITY_ATTRIBUTES lpTokenAttributes, TOKEN_TYPE TokenType, SECURITY_IMPERSONATION_LEVEL ImpersonationLevel)
{
    HANDLE hToken = NULL;

    //     
    BOOL bRet = LogonUser(lpszUsername, lpszDomain, lpszPassword, dwLogonType, LOGON32_PROVIDER_DEFAULT, &hToken);

    if (FALSE != bRet)
    {
        HANDLE hNewToken = NULL;

        //   
        bRet = DuplicateTokenEx(hToken, dwDesiredAccess, lpTokenAttributes, ImpersonationLevel, TokenType, &hNewToken);

        //     
        CloseHandle(hToken);

        //  
        hToken = (FALSE != bRet) ? hNewToken : NULL;
    } // if

    return hToken;
} // OpenUserToken

// ------------------------------------------------------------------------------------------------
BOOL GetTokenUser(HANDLE hToken, PSID *ppSid)
{
    DWORD cb;

    //     ( )
    GetTokenInformation(hToken, TokenUser, NULL, 0, &cb);

    //   
    PTOKEN_USER pTokenUser = (PTOKEN_USER)LocalAlloc(LPTR, cb);
    if (NULL == pTokenUser) return FALSE;

    //       
    BOOL bRet = GetTokenInformation(hToken, TokenUser, pTokenUser, cb, &cb);

    PSID pSid = NULL;

    //  SID 
    if (FALSE != bRet)
    {
        DWORD cbSid = GetLengthSid(pTokenUser->User.Sid);
        pSid = (PSID)LocalAlloc(LPTR, cbSid);

        bRet = CopySid(cbSid, pSid, pTokenUser->User.Sid);
    } // if

    //   
    LocalFree(pTokenUser);

    if (FALSE != bRet)
    {
        *ppSid = pSid; //  
    } // if
    else
    {
        LocalFree(pSid); //   
    } // else

    return bRet;
} // GetTokenUser

// ------------------------------------------------------------------------------------------------
BOOL GetTokenGroups(HANDLE hToken, PTOKEN_GROUPS *pTokenGroups)
{
    DWORD cb;

    //     ( )
    GetTokenInformation(hToken, TokenGroups, NULL, 0, &cb);

    //   
    PTOKEN_GROUPS Groups = (PTOKEN_GROUPS)LocalAlloc(LPTR, cb);
    if (NULL == Groups) return FALSE;

    //      
    BOOL bRet = GetTokenInformation(hToken, TokenGroups, Groups, cb, &cb);

    if (FALSE != bRet)
    {
        *pTokenGroups = Groups; //  
    } // if
    else
    {
        LocalFree(Groups); //   
    } // else

    return bRet;
} // GetTokenGroups

// ------------------------------------------------------------------------------------------------
BOOL GetTokenPrivileges(HANDLE hToken, PTOKEN_PRIVILEGES *pTokenPrivileges)
{
    DWORD cb;

    //     ( )
    GetTokenInformation(hToken, TokenPrivileges, NULL, 0, &cb);

    //   
    PTOKEN_PRIVILEGES Privileges = (PTOKEN_PRIVILEGES)LocalAlloc(LPTR, cb);
    if (NULL == Privileges) return FALSE;

    //      
    BOOL bRet = GetTokenInformation(hToken, TokenPrivileges, Privileges, cb, &cb);

    if (FALSE != bRet)
    {
        *pTokenPrivileges = Privileges; //  
    } // if
    else
    {
        //   
        LocalFree(Privileges);
    } // else

    return bRet;
} // GetTokenPrivileges

// ------------------------------------------------------------------------------------------------
BOOL IsPrivilegeExists(HANDLE hToken, LPCTSTR lpszPrivilegeName)
{
    LUID Luid;
    PTOKEN_PRIVILEGES pTokenPrivileges = NULL;

    //  LUID  
    BOOL bRet = LookupPrivilegeValue(NULL, lpszPrivilegeName, &Luid);

    //   
    if (FALSE != bRet)
        bRet = GetTokenPrivileges(hToken, &pTokenPrivileges);

    if (FALSE == bRet) return -1; //  

    bRet = FALSE;
    for (DWORD i = 0; i < pTokenPrivileges->PrivilegeCount; ++i)
    {
        //   LUID
        int res = memcmp(&pTokenPrivileges->Privileges[i].Luid, &Luid, sizeof(LUID));

        if (0 == res)
        {
            bRet = TRUE;
            break; //   
        } // if
    } // for

    //   
    LocalFree(pTokenPrivileges);

    return bRet;
} // IsPrivilegeExists

// ------------------------------------------------------------------------------------------------
BOOL IsPrivilegesExists(HANDLE hToken, LPCTSTR names[], DWORD dwCount, BOOL bAllPrivileges)
{
    LUID Luid;
    PTOKEN_PRIVILEGES pTokenPrivileges = NULL;

    //   
    BOOL bRet = GetTokenPrivileges(hToken, &pTokenPrivileges);
    if (FALSE == bRet) return -1; //  

    for (DWORD i = 0; i < dwCount; ++i)
    {
        //  LUID  
        bRet = LookupPrivilegeValue(NULL, names[i], &Luid);

        if (FALSE == bRet) //  
        {
            bRet = -1;
            break; //   
        } // if

        bRet = FALSE;

        for (DWORD j = 0;
            j < pTokenPrivileges->PrivilegeCount; ++j)
        {
            //   LUID
            int res = memcmp(&pTokenPrivileges->Privileges[j].Luid, &Luid, sizeof(LUID));

            if (0 == res)
            {
                bRet = TRUE;
                break; //   
            } // if
        } // for

        if ((FALSE == bRet && FALSE != bAllPrivileges) ||
            (FALSE != bRet && FALSE == bAllPrivileges))
        {
            break; //   
        } // if
    } // for

    //   
    LocalFree(pTokenPrivileges);

    return bRet;
} // IsPrivilegesExists

// ------------------------------------------------------------------------------------------------
BOOL IsPrivilegeEnabled(HANDLE hToken, LPCTSTR lpszPrivilegeName)
{
    BOOL fResult; //  
    PRIVILEGE_SET PrivSet;

    PrivSet.PrivilegeCount = 1; //  
    PrivSet.Control = PRIVILEGE_SET_ALL_NECESSARY;

    //  LUID  
    BOOL bRet = LookupPrivilegeValue(NULL, lpszPrivilegeName, &PrivSet.Privilege[0].Luid);

    if (FALSE != bRet)
    {
        //    
        bRet = PrivilegeCheck(hToken, &PrivSet, &fResult);
    } // if

    return (FALSE != bRet) ? fResult : -1;
} // IsPrivilegeEnabled

// ------------------------------------------------------------------------------------------------
BOOL PrivilegesCheck(HANDLE hToken, LPCTSTR names[], DWORD dwCount, BOOL bSetAllNecessary, PPRIVILEGE_SET *pPrivSet)
{
    //     ( )
    //   
    DWORD cb = sizeof(PRIVILEGE_SET) + (dwCount - 1) * sizeof(LUID_AND_ATTRIBUTES);

    //      
    PPRIVILEGE_SET PrivSet = (PPRIVILEGE_SET)LocalAlloc(LPTR, cb);
    if (NULL == PrivSet) return -1; //  

    BOOL fResult; //  

    //    
    PrivSet->PrivilegeCount = dwCount;
    //        
    PrivSet->Control = (FALSE != bSetAllNecessary) ? PRIVILEGE_SET_ALL_NECESSARY : 0;

    BOOL bRet = TRUE;

    //  LUID     
    for (DWORD i = 0; i < dwCount && FALSE != bRet; ++i)
    {
        bRet = LookupPrivilegeValue(NULL, names[i], &PrivSet->Privilege[i].Luid);
    } // for

    if (FALSE != bRet)
    {
        //    
        bRet = PrivilegeCheck(hToken, PrivSet, &fResult);
    } // if

    if (FALSE != bRet && NULL != pPrivSet)
    {
        *pPrivSet = PrivSet; //  
    } // if
    else
    {
        //   
        LocalFree(PrivSet);
    } // else

    return (FALSE != bRet) ? fResult : -1;
} // PrivilegesCheck

// ------------------------------------------------------------------------------------------------
BOOL EnablePrivilege(HANDLE hToken, LPCTSTR lpszPrivilegeName, BOOL bEnable)
{
    TOKEN_PRIVILEGES Privileges;
    Privileges.PrivilegeCount = 1; //  

    //     
    Privileges.Privileges[0].Attributes = (FALSE != bEnable) ? SE_PRIVILEGE_ENABLED : 0;

    //  LUID  
    BOOL bRet = LookupPrivilegeValue(NULL, lpszPrivilegeName, &Privileges.Privileges[0].Luid);

    if (FALSE != bRet)
    {
        //    
        bRet = AdjustTokenPrivileges(hToken, FALSE, &Privileges, sizeof(TOKEN_PRIVILEGES), NULL, NULL);
    } // if

    return bRet;
} // EnablePrivilege

// ------------------------------------------------------------------------------------------------
BOOL AdjustPrivileges(HANDLE hToken, LPCTSTR names[], DWORD NewState[], DWORD dwCount)
{
    //     ( )
    DWORD cb = sizeof(TOKEN_PRIVILEGES) + (dwCount - 1) * sizeof(LUID_AND_ATTRIBUTES);

    //   
    PTOKEN_PRIVILEGES Privileges = (PTOKEN_PRIVILEGES)LocalAlloc(LPTR, cb);
    if (NULL == Privileges) return FALSE;

    BOOL bRet = TRUE;

    //    
    Privileges->PrivilegeCount = dwCount;

    //  LUID    
    //     
    for (DWORD i = 0; i < dwCount && FALSE != bRet; ++i)
    {
        Privileges->Privileges[i].Attributes = NewState[i];
        bRet = LookupPrivilegeValue(NULL, names[i], &Privileges->Privileges[i].Luid);
    } // for

    if (FALSE != bRet)
    {
        //    
        bRet = AdjustTokenPrivileges(hToken, FALSE, Privileges, cb, NULL, NULL);
    } // if

    //   
    LocalFree(Privileges);

    return bRet;
} // AdjustPrivileges

// ------------------------------------------------------------------------------------------------
BOOL DisableAllPrivileges(HANDLE hToken)
{
    return AdjustTokenPrivileges(hToken, TRUE, NULL, 0, NULL, NULL);
} // DisableAllPrivileges
