// sid.cpp
//
// Copyright (c), ..  (andrei.glukhoedov@gmail.com)
//

#include <Windows.h>
#include <tchar.h>
#include <sddl.h>
#include <strsafe.h>

// ------------------------------------------------------------------------------------------------
BOOL GetAccountSID(LPCTSTR lpAccountName, PSID *ppSid)
{
    PSID pSid = NULL;
    LPTSTR lpszDomainName = NULL;

    DWORD cbSid = 0, cchDomainName = 0;

    //   
    LookupAccountName(NULL, lpAccountName, NULL, &cbSid, NULL, &cchDomainName, NULL);

    if (cbSid > 0)
    {
        //       SID
        pSid = (PSID)LocalAlloc(LMEM_FIXED, cbSid);
    } // if

    if (cchDomainName > 0)
    {
        //        
        lpszDomainName = (LPTSTR)LocalAlloc(LMEM_FIXED, cchDomainName*sizeof(TCHAR));
    } // if

    BOOL bRet = FALSE;

    if (NULL != pSid && NULL != lpszDomainName)
    {
        SID_NAME_USE SidType;

        //  SID   
        bRet = LookupAccountName(NULL, lpAccountName, pSid, &cbSid, lpszDomainName, &cchDomainName, &SidType);
    } // if

    if (FALSE != bRet)
    {
        *ppSid = pSid; //   SID
    } // if
    else if (NULL != pSid)
    {
        LocalFree(pSid); //   
    } // if

    //   
    if (NULL != lpszDomainName) LocalFree(lpszDomainName);

    return bRet;
} // GetAccountSID

// ------------------------------------------------------------------------------------------------
BOOL GetWellKnownSID(WELL_KNOWN_SID_TYPE WellKnownSidType, PSID pDomainSid, PSID *ppSid)
{
    DWORD cbSid = SECURITY_MAX_SID_SIZE;

    //       SID
    PSID pSid = (PSID)LocalAlloc(LMEM_FIXED, cbSid);
    if (NULL == pSid) return FALSE;

    //  SID 
    BOOL bRet = CreateWellKnownSid(WellKnownSidType, pDomainSid, pSid, &cbSid);

    if (FALSE != bRet)
    {
        *ppSid = pSid; //   SID
    } // if
    else
    {
        LocalFree(pSid); //   
    } // else

    return bRet;
} // GetWellKnownSID

// ------------------------------------------------------------------------------------------------
BOOL GetAccountName(PSID pSid, LPTSTR *lpName)
{
    //   SID
    if (FALSE == IsValidSid(pSid)) return FALSE;

    LPTSTR lpszName = NULL;
    DWORD cch = 0, cchDomainName = 0;

    //   
    LookupAccountSid(NULL, pSid, NULL, &cch, NULL, &cchDomainName, NULL);
    DWORD cb = (cch + cchDomainName) * sizeof(TCHAR);

    //     
    if (cb > 0) lpszName = (LPTSTR)LocalAlloc(LMEM_FIXED, cb);

    BOOL bRet = FALSE;
    SID_NAME_USE SidType;

    if (NULL != lpszName)
    {
        //    
        bRet = LookupAccountSid(NULL, pSid, lpszName + cchDomainName, &cch, lpszName, &cchDomainName, &SidType);
    } // if

    if (FALSE != bRet)
    {
        if (SidTypeDomain != SidType)
        {
            if (cchDomainName > 0) lpszName[cchDomainName] = TEXT('\\');
            else StringCbCopy(lpszName, cb, lpszName + 1);
        } // if

        *lpName = lpszName; //   
    } // if
    else
    {
        //     ,  SID  
        ConvertSidToStringSid(pSid, lpName);

        //   
        if (NULL != lpszName) LocalFree(lpszName);
    } // else

    return bRet;
} // GetAccountName
