// Clipboard.cpp
//
// Copyright (c), ..  (andrei.glukhoedov@gmail.com)
//

#include <Windows.h>
#include <tchar.h>
#include <strsafe.h>

#include "SampleIPCLibrary.h"

// ------------------------------------------------------------------------------------------------
BOOL WriteClipboard(LPCTSTR lpszText)
{
    BOOL bRet = OpenClipboard(NULL); //   

    if (FALSE != bRet)
    {
        EmptyClipboard(); //    

        //   ( )  
        DWORD cb = (_tcslen(lpszText) + 1) * sizeof(TCHAR);

        //     
        HANDLE hMem = GlobalAlloc(GMEM_MOVEABLE, cb);
        bRet = (NULL != hMem) ? TRUE : FALSE;

        if (FALSE != bRet)
        {
            //     
            StringCbCopy((LPTSTR)GlobalLock(hMem), cb, lpszText);
            GlobalUnlock(hMem);

#ifdef UNICODE
            UINT uFormat = CF_UNICODETEXT;
#else
            UINT uFormat = CF_TEXT;
#endif /* UNICODE */

            //     
            HANDLE hRet = SetClipboardData(uFormat, hMem);
            bRet = (NULL != hRet) ? TRUE : FALSE;
        } // if

        CloseClipboard(); //   

        //   
        if (NULL != hMem) GlobalFree(hMem);
    } // if

    return bRet;
} // WriteClipboard

// ------------------------------------------------------------------------------------------------
BOOL ReadClipboard(LPTSTR lpszText, DWORD cch)
{
#ifdef UNICODE
    UINT uFormat = CF_UNICODETEXT;
#else
    UINT uFormat = CF_TEXT;
#endif /* UNICODE */

    //         
    BOOL bRet = IsClipboardFormatAvailable(uFormat);

    if (FALSE != bRet) //  
        bRet = OpenClipboard(NULL); //   

    if (FALSE != bRet)
    {
        //    
        HANDLE hMem = GetClipboardData(uFormat);
        bRet = (NULL != hMem) ? TRUE : FALSE;

        if (FALSE != bRet)
        {
            //      
            StringCchCopy(lpszText, cch, (LPCTSTR)GlobalLock(hMem));
            GlobalUnlock(hMem);
        } // if

        CloseClipboard(); //   
    } // if

    return bRet;
} // ReadClipboard

// ------------------------------------------------------------------------------------------------
BOOL WriteClipboard(LPCTSTR lpszFormat, LPCVOID lpData, DWORD cb)
{
    //   
    UINT uFormat = RegisterClipboardFormat(lpszFormat);
    //   
    BOOL bRet = (0 != uFormat) ? OpenClipboard(NULL) : FALSE;

    if (FALSE != bRet)
    {
        EmptyClipboard(); //    

        //     
        HANDLE hMem = GlobalAlloc(GMEM_MOVEABLE, cb);
        bRet = (NULL != hMem) ? TRUE : FALSE;

        if (FALSE != bRet)
        {
            //     
            CopyMemory(GlobalLock(hMem), lpData, cb);
            GlobalUnlock(hMem);

            //     
            HANDLE hRet = SetClipboardData(uFormat, hMem); 
            bRet = (NULL != hRet) ? TRUE : FALSE;
        } // if

        CloseClipboard(); //   

        //   
        if (NULL != hMem) GlobalFree(hMem); 
    } // if

    return bRet;
} // WriteClipboard

// ------------------------------------------------------------------------------------------------
BOOL ReadClipboard(LPCTSTR lpszFormat, LPVOID lpData, DWORD cb, LPDWORD lpcbNeeded)
{
    //   
    UINT uFormat = RegisterClipboardFormat(lpszFormat);

    //        
    BOOL bRet = (0 != uFormat) ? IsClipboardFormatAvailable(uFormat) : FALSE;

    if (FALSE != bRet) //  
        bRet = OpenClipboard(NULL); //   

    if (FALSE != bRet)
    {
        //    
        HANDLE hMem = GetClipboardData(uFormat);
        bRet = (NULL != hMem) ? TRUE : FALSE;

        if (FALSE != bRet)
        {
            //   ( ) 
            DWORD cbMem = GlobalSize(hMem);

            if (NULL != lpData)
            {
                //      
                CopyMemory(lpData, GlobalLock(hMem), min(cb, cbMem));
                GlobalUnlock(hMem);
            } // if

            //    
            if (NULL != lpcbNeeded) *lpcbNeeded = cbMem;
        } // if

        CloseClipboard(); //   
    } // if

    return bRet;
} // ReadClipboard
